# -*- coding: utf-8 -*-
"""
Django admin integration for integrated_channel app.
"""
from __future__ import absolute_import, unicode_literals

from django.contrib import admin

from integrated_channels.integrated_channel.models import (
    EnterpriseCustomerPluginConfiguration, EnterpriseIntegratedChannel
)


@admin.register(EnterpriseIntegratedChannel)
class EnterpriseIntegratedChannelAdmin(admin.ModelAdmin):
    """
    Django admin model for EnterpriseIntegratedChannel.
    """
    list_display = (
        "name",
        "data_type",
    )

    list_filter = ("data_type",)
    search_fields = ("name", "data_type",)

    class Meta(object):
        model = EnterpriseIntegratedChannel



# -*- coding: utf-8 -*-
"""
Database models for Enterprise Integrated Channel SAP SuccessFactors.
"""

from __future__ import absolute_import, unicode_literals

import json
from config_models.models import ConfigurationModel
from simple_history.models import HistoricalRecords

from django.db import models
from django.utils.encoding import python_2_unicode_compatible

from model_utils.models import TimeStampedModel

from integrated_channels.integrated_channel.models import EnterpriseCustomerPluginConfiguration
from integrated_channels.sap_success_factors.utils import SapCourseExporter, parse_datetime_to_epoch
from integrated_channels.integrated_channel.learner_data import BaseLearnerExporter

from integrated_channels.sap_success_factors.transmitters.courses import SuccessFactorsCourseTransmitter
from integrated_channels.sap_success_factors.transmitters.learner_data import SuccessFactorsLearnerDataTransmitter


@python_2_unicode_compatible
class SAPSuccessFactorsGlobalConfiguration(ConfigurationModel):
    """
    The Global configuration for integrating with SuccessFactors.
    """

    completion_status_api_path = models.CharField(max_length=255)
    course_api_path = models.CharField(max_length=255)
    oauth_api_path = models.CharField(max_length=255)
    provider_id = models.CharField(max_length=100, default='EDX')

    class Meta:
        app_label = 'sap_success_factors'

    def __str__(self):
        """
        Return human-readable string representation.
        """
        return "<SAPSuccessFactorsGlobalConfiguration with id {id}>".format(
            id=self.id
        )

    def __repr__(self):
        """
        Return uniquely identifying string representation.
        """
        return self.__str__()


@python_2_unicode_compatible
class SAPSuccessFactorsEnterpriseCustomerConfiguration(EnterpriseCustomerPluginConfiguration):
    """
    The Enterprise specific configuration we need for integrating with SuccessFactors.
    """

    USER_TYPE_USER = 'user'
    USER_TYPE_ADMIN = 'admin'

    USER_TYPE_CHOICES = (
        (USER_TYPE_USER, 'User'),
        (USER_TYPE_ADMIN, 'Admin'),
    )

    key = models.CharField(max_length=255, blank=True, verbose_name="Client ID")
    sapsf_base_url = models.CharField(max_length=255, verbose_name="SAP Base URL")
    sapsf_company_id = models.CharField(max_length=255, blank=True, verbose_name="SAP Company ID")
    sapsf_user_id = models.CharField(max_length=255, blank=True, verbose_name="SAP User ID")
    secret = models.CharField(max_length=255, blank=True, verbose_name="Client Secret")
    user_type = models.CharField(
        max_length=20,
        choices=USER_TYPE_CHOICES,
        blank=False,
        default=USER_TYPE_USER,
        verbose_name="SAP User Type"
    )

    history = HistoricalRecords()

    class Meta:
        app_label = 'sap_success_factors'

    def __str__(self):
        """
        Return human-readable string representation.
        """
        return "<SAPSuccessFactorsEnterpriseCustomerConfiguration for Enterprise {enterprise_name}>".format(
            enterprise_name=self.enterprise_customer.name
        )

    def __repr__(self):
        """
        Return uniquely identifying string representation.
        """
        return self.__str__()

    @staticmethod
    def channel_code():
        """
        Returns an capitalized identifier for this channel class, unique among subclasses.
        """
        return 'SAP'

    def get_learner_data_record(self, enterprise_enrollment, completed_date=None, grade=None, is_passing=False):
        """
        Returns a LearnerDataTransmissionAudit initialized from the given enrollment and course completion data.

        If completed_date is None, then course completion has not been met.
        """
        # Have to create the audit model instance here to avoid a circular dependency.
        completed_timestamp = None
        course_completed = False
        if completed_date is not None:
            completed_timestamp = parse_datetime_to_epoch(completed_date)
            course_completed = is_passing

        return LearnerDataTransmissionAudit(
            enterprise_course_enrollment_id=enterprise_enrollment.id,
            sapsf_user_id=enterprise_enrollment.enterprise_customer_user.get_remote_id(),
            course_id=enterprise_enrollment.course_id,
            course_completed=course_completed,
            completed_timestamp=completed_timestamp,
            grade=grade,
        )

    def get_learner_data_exporter(self, user):
        """
        Returns a SAP learner data exporter instance.
        """
        return BaseLearnerExporter(user, self)

    def get_learner_data_transmitter(self):
        """
        Returns a SuccessFactorsLearnerDataTransmitter instance.
        """
        return SuccessFactorsLearnerDataTransmitter(self)

    def get_course_data_exporter(self, user):
        """
        Returns a SapCourseExporter instance.
        """
        return SapCourseExporter(user, self)

    def get_course_data_transmitter(self):
        """
        Returns a SuccessFactorsCourseTransmitter instance.
        """
        return SuccessFactorsCourseTransmitter(self)


@python_2_unicode_compatible
class LearnerDataTransmissionAudit(models.Model):
    """
    The payload we sent to SuccessFactors at a given point in time for an enterprise course enrollment.
    """

    enterprise_course_enrollment_id = models.PositiveIntegerField(blank=False, null=False)
    sapsf_user_id = models.CharField(max_length=255, blank=False, null=False)
    course_id = models.CharField(max_length=255, blank=False, null=False)
    course_completed = models.BooleanField(default=True)
    completed_timestamp = models.BigIntegerField()  # we send a UNIX timestamp to SuccessFactors
    instructor_name = models.CharField(max_length=255, blank=True)
    grade = models.CharField(max_length=100, blank=False, null=False)
    status = models.CharField(max_length=100, blank=False, null=False)
    error_message = models.TextField(blank=True)
    created = models.DateTimeField(auto_now_add=True)

    class Meta:
        app_label = 'sap_success_factors'

    def __str__(self):
        """
        Return human-readable string representation.
        """
        return '<LearnerDataTransmissionAudit {} for enterprise enrollment {}, SAP user {}, and course {}>'.format(
            self.id,
            self.enterprise_course_enrollment_id,
            self.sapsf_user_id,
            self.course_id
        )

    def __repr__(self):
        """
        Return uniquely identifying string representation.
        """
        return self.__str__()

    @property
    def provider_id(self):
        '''
        Fetch ``provider_id`` from global configuration settings
        '''
        return SAPSuccessFactorsGlobalConfiguration.current().provider_id

    def _payload_data(self):
        """
        Convert the audit record's fields into SAP SuccessFactors key/value pairs.
        """
        return dict(
            userID=self.sapsf_user_id,
            courseID=self.course_id,
            providerID=self.provider_id,
            # SAP SuccessFactors requires strings, not boolean values.
            courseCompleted="true" if self.course_completed else "false",
            completedTimestamp=self.completed_timestamp,
            grade=self.grade,
            # TODO: determine these values from the enrollment seat?
            # We might at a later date choose to send these values,
            # but at this point we will omit them from the payload to avoid errors.
            #
            # price=empty_value,
            # currency=empty_value,
            # creditHours=empty_value,
            # totalHours=empty_value,
            # contactHours=empty_value,
            # cpeHours=empty_value,
            # instructorName=empty_value,
            # comments=empty_value,
        )

    def serialize(self):
        """
        Return a JSON-serialized representation.

        Sort the keys so the result is consistent and testable.
        """
        return json.dumps(self._payload_data(), sort_keys=True)


@python_2_unicode_compatible
class CatalogTransmissionAudit(TimeStampedModel):
    """
    The summary of instances when the course catalog was sent to SuccessFactors for an enterprise.
    """

    enterprise_customer_uuid = models.UUIDField(blank=False, null=False)
    total_courses = models.PositiveIntegerField(blank=False, null=False)
    status = models.CharField(max_length=100, blank=False, null=False)
    error_message = models.TextField(blank=True)
    audit_summary = models.TextField(default='{}')

    class Meta:
        app_label = 'sap_success_factors'

    def __str__(self):
        """
        Return human-readable string representation.
        """
        return "<CatalogTransmissionAudit {} for Enterprise {}> for {} courses>".format(
            self.id,
            self.enterprise_customer_uuid,
            self.total_courses
        )

    def __repr__(self):
        """
        Return uniquely identifying string representation.
        """
        return self.__str__()



# -*- coding: utf-8 -*-
"""
Class for transmitting course data to SuccessFactors.
"""
from __future__ import absolute_import, unicode_literals

import logging
import json
from django.apps import apps
from django.core.exceptions import ObjectDoesNotExist

from integrated_channels.sap_success_factors.transmitters import SuccessFactorsTransmitterBase
from requests import RequestException


LOGGER = logging.getLogger(__name__)


class SuccessFactorsCourseTransmitter(SuccessFactorsTransmitterBase):
    """
    This endpoint is intended to carry out an export of course data to SuccessFactors for a given Enterprise.
    """

    def transmit_block(self, serialized_payload):
        """
        SAPSuccessFactors can only send 1000 items at a time, so this method sends one "page" at a time.

        Args:
            serialized_payload (bytes): A set of bytes containing a page's worth of data

        Returns:
            status_code (int): An integer status for the HTTP request
            body (str): The SAP SuccessFactors server's response body
        """
        LOGGER.info(serialized_payload)
        try:
            status_code, body = self.client.send_course_import(serialized_payload)
        except RequestException as request_exception:
            status_code = 500
            body = str(request_exception)

        if status_code >= 400:
            LOGGER.error('Failed to send course metadata for Enterprise Customer {}\nError Message {}'.
                         format(self.enterprise_configuration.enterprise_customer.name, body))
        else:
            LOGGER.debug('Successfully sent course metadata for Enterprise Customer {}'.
                         format(self.enterprise_configuration.enterprise_customer.name))

        return status_code, body

    def transmit(self, payload):
        """
        Send a course data import call to SAP SuccessFactors using the client.

        Args:
            payload (SapCourseExporter): The OCN course exporter object to send to SAP SuccessFactors
        """
        CatalogTransmissionAudit = apps.get_model(  # pylint: disable=invalid-name
            app_label='sap_success_factors',
            model_name='CatalogTransmissionAudit'
        )

        try:
            last_catalog_transmission = CatalogTransmissionAudit.objects.filter(
                error_message='',
                enterprise_customer_uuid=self.enterprise_configuration.enterprise_customer.uuid
            ).latest('created')
        except ObjectDoesNotExist:
            # This should happen if there have been 0 successful catalog transmissions for this enterprise
            last_audit_summary = {}
        else:
            last_audit_summary = json.loads(last_catalog_transmission.audit_summary)

        audit_summary = payload.resolve_removed_courses(last_audit_summary)

        total_transmitted = 0
        errors = []
        status_codes = []
        for serialized_payload, length in payload.get_serialized_data_blocks():
            status_code, body = self.transmit_block(serialized_payload)
            status_codes.append(str(status_code))
            error_message = body if status_code >= 400 else ''
            if error_message:
                errors.append(error_message)
            else:
                total_transmitted += length

        error_message = ', '.join(errors) if errors else ''
        code_string = ', '.join(status_codes)

        catalog_transmission_audit = CatalogTransmissionAudit(
            enterprise_customer_uuid=self.enterprise_configuration.enterprise_customer.uuid,
            total_courses=len(payload.courses),
            status=code_string,
            error_message=error_message,
            audit_summary=json.dumps(audit_summary),
        )

        catalog_transmission_audit.save()
        return catalog_transmission_audit



# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-02-24 15:08
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='EnterpriseIntegratedChannel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('name', models.CharField(help_text='Third Party name.', max_length=255)),
                ('data_type', models.CharField(help_text='Data Type', max_length=100)),
            ],
            options={
                'verbose_name': 'Enterprise Integrated Channel',
                'verbose_name_plural': 'Enterprise Integrated Channels',
            },
        ),
        migrations.AlterUniqueTogether(
            name='enterpriseintegratedchannel',
            unique_together=set([('name', 'data_type')]),
        ),
    ]



"""
Integrated channel package for Enterprises.

Integrated channels are external systems that manage Enterprise Customer data. This package contains logic
for configuring integrated channels and sending data between us and an integrated channel.
"""



# -*- coding: utf-8 -*-
"""
Decorators for Enterprise API views.
"""
from __future__ import absolute_import, unicode_literals

from functools import wraps

from rest_framework.exceptions import PermissionDenied

from enterprise.utils import get_enterprise_customer_for_user


def enterprise_customer_required(view):
    """
    Ensure the user making the API request is associated with an EnterpriseCustomer.

    This decorator attempts to find an EnterpriseCustomer associated with the requesting
    user and passes that EnterpriseCustomer to the view as a parameter. It will return a
    PermissionDenied error if an EnterpriseCustomer cannot be found.

    Usage::
        @enterprise_customer_required()
        def my_view(request, enterprise_customer):
            # Some functionality ...

        OR

        class MyView(View):
            ...
            @method_decorator(enterprise_customer_required)
            def get(self, request, enterprise_customer):
                # Some functionality ...
    """
    @wraps(view)
    def wrapper(request, *args, **kwargs):
        """
        Checks for an enterprise customer associated with the user, calls the view function
        if one exists, raises PermissionDenied if not.
        """
        user = request.user
        enterprise_customer = get_enterprise_customer_for_user(user)
        if enterprise_customer:
            args = args + (enterprise_customer,)
            return view(request, *args, **kwargs)
        else:
            raise PermissionDenied(
                'User {username} is not associated with an EnterpriseCustomer.'.format(
                    username=user.username
                )
            )
    return wrapper
